# chef_hostname Cookbook

Sets the node's hostname

* resource-driven cookbook
* supports FQDNs as hostnames
* persists after a reboot
* reloads ohai
* runs at compile-time (no need to use lazy)
* fixes up /etc/hosts so node["fqdn"] works
* runs nearly everywhere
* supports hostnamectl from systemd

## Motivation

* Make strong guarantees that `node["fqdn"]` in other recipes "just works" without having to edit them to use `lazy { node["fqdn"] }`
* Be very portable

## Requirements

## Platforms
- Ubuntu/Debian/Mint/Raspbian
- RHEL/CentOS/Scientific/Oracle/Fedora/Pidora
- OpenSUSE/SLES
- FreeBSD/OpenBSD/NetBSD
- Windows

### Chef
- Chef 12.1+

### Cookbooks
- compat_resource


## Custom Resources

hostname Sets the hostname, ensures that reboot will preserve the hostname, re-runs the ohai plugin to set the node data.

### Actions

- :set: Ses the hostname

### Properties

- hostname: hostname to set
- compile_time:  defaults to running at compile time, set to false to disable

## Examples

Setting hostname to a string:

```ruby
hostname "foo.example.com"
```

Setting hostname to the node name:

```ruby
hostname node.name
```

Setting hostname to whatever attribute you like:

```ruby
hostname node['set_fqdn']
```

There is no need to "lazy" arguments to templates and filenames when this is used since it forces itself to run at compile-time.

```ruby
hostname node.name

# node["fqdn"] will be set here at compile time
template "/etc/motd" do
  source "motd.erb"
  variables({
    fqdn: node["fqdn"]
  })
end

# /bin/hostname will be set here at compile time
myhostname = `/bin/hostname`

file "/etc/issue" do
  content myhostname
end
```

The hostname resource will drop a line into /etc/hosts so that the `node["fqdn"]` can be resolved correctly, and will re-trigger ohai.  The default is
to use the node["ipaddress"]` value for the ipaddress on the /etc/hosts line.  In order to override it:

```ruby
hostname node["cloud"]["public_hostname"]
  ipaddress node["cloud"]["public_ipv4"]
end
```

In order to override the editing of the /etc/hosts file pass nil for the ipaddress (note that if you edit the /etc/hosts file you will be responsible
for also reloading the ohai plugin and you will want to do both at compile-time yourself in order for `node["fqdn"]` to resolve)

```ruby
hostname node.name
  ipaddress nil
end
```

Aliases can also be added to the line that hostname adds to /etc/hosts:

```ruby
hostname node.name
  ipaddress "259.1.1.1"
  aliases [ "klowns.car.local", "britney" ]
end
```

## Notes

There are no recipes in this cookbook, the resource is meant to be used in your own custom recipes.  There are no attributes in this cookbook,
you can drive the resource off of whatever attribute(s) you like.

## TODO
- fix setting node['fqdn'] correctly on windows
- mac
- solaris
- aix
- nexus
- arch
- gentoo

## License & Authors

```
Author:: Lamont Granquist (<lamont@chef.io>)

Copyright:: 2016-2016, Chef Software, Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
```
